local MAJOR = "StarLibBuffer-1.0"
local MINOR = 1
assert(LibStub, MAJOR.." requires LibStub") 
local LibBuffer = LibStub:NewLibrary(MAJOR, MINOR)
if not LibBuffer then return end
local LibError = LibStub("StarLibError-1.0")
assert(LibError, MAJOR .. " requires StarLibError-1.0")

local pool = setmetatable({}, {__mode = "k"})

local update
local frame = CreateFrame("Frame")

local DEFAULT_LIMIT = 50

if not LibBuffer.__index then
	LibBuffer.__index = LibBuffer
end

-- @name StarLibBuffer:New
-- @usage StarLibBuffer:New(name, size, defval, errorLevel)
-- @param name A name for the buffer
-- @param size The buffer's size
-- @param defval The buffer's default value. The alpha channel.
-- @param errorLevel The verbocity level
-- @return A new StarLibBuffer object

function LibBuffer:New(name, size, defval, errorLevel)
	assert(type(name) ~= "nil", "LibBuffer requires a name")
	
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
		obj.buffer = {}
	end

	setmetatable(obj, self)

	obj:Resize(size)
	obj.name = name
	obj.defval = defval
	obj.error = LibError:New(MAJOR .. " " .. name, errorLevel)

	return obj	
	
end

-- @name StarLibBuffer:Del
-- @usage StarLibBuffer:Del([lcd]) or object:Del()
-- @param lcd An optional lcd object
-- @return Nothing
function LibBuffer:Del(buff)
	if not buff then buff = self end
	pool[buff] = true
	buff:Clear()
	buff.error:Del()
end

function LibBuffer:Resize(size)
	if self:Size() < size then
		for i = self:Size() - 1, size - 1 do
			self.buffer[i] = self.defval
		end
	end
	if self:Size() > size then
		for i = size - 1, self.size - 1 do
			self.buffer[i] = false
		end
	end
	self.size = size
end

function LibBuffer:Replace(pos, val)
	if pos >= self.size then return end
	if type(val) == "nil" or val == "" then
		val = ' '
	end
	if type(val) ~= "string" then
		self.buffer[pos] = val
	else
		self.buffer[pos] = val:sub(1, 1)
	end
end

function LibBuffer:Clear()
	for i = 0, self.size - 1 do
		self.buffer[i] = self.defval
	end
end

function LibBuffer:AsString()
	if type(self.defval) ~= "string" then return end
	local str = ""
	for i = 0, self:Size() - 1 do
		if type(self.buffer[i]) == "string" then
			str = str .. self.buffer[i]
		else
			self.error:Print(":AsString() - Bad index: " .. i .. ", type: " .. type(self.buffer[i]) .. ", size: " .. self:Size())
			break
		end
	end
	return str
end

function LibBuffer:FromString(str)
	if type(str) ~= "string" then return end
	for i = 0, strlen(str) - 1 do
		self.buffer[i] = str:sub(i + 1, i + 1)
	end
	self.size = strlen(str)
end

function LibBuffer:Size()
	--[[
	local count = 0
	for k in pairs(self.buffer) do
		count = count + 1
	end
	return count
	]]
	return self.size or 0
end

function LibBuffer:Sub(num1, num2)

	if not num2 then num2 = self:Size() - 1 end
	
	local j = self:Size() - num1 + (num2 - num1)
	local t = LibBuffer:New(self.name .. ":" , j, self.defval, self.errorLevel)
	for i = num1, num2 do
		t.buffer[i - num1] = self.buffer[i]
	end
	return t
end

function LibBuffer:Copy(other)
	table.wipe(self.buffer)
	for k, v in pairs(other.buffer) do
		self.buffer[k] = other.buffer[k]
	end
end

function LibBuffer:Merge(from, pos, len)
	local i = 0
	while i < len - (self.size - len) do
		self.buffer[i + pos] = from.buffer[i]
		i = i + 1
	end
end

function LibBuffer:Memset(pos, char, length)
	for n = pos, pos + length - 1 do
		self.buffer[n] = char
	end
end

function LibBuffer:Memcpy(pos1, buffer, pos2, length)
	for n = 0, length - 1 do
		self.buffer[pos1 + n] = buffer.buffer[pos2 + n]
	end
end
